###############################################################################
# zapplet - A tray applet for interfacing with Zenoss
# Copyright (C) 2008 Nathaniel McCallum <nathaniel@natemccallum.com>
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
###############################################################################

import gtk

import zapplet.os

class PreferencesWindow(gtk.Window):
	"""This is the preferences window, which is a GUI to the Config classes."""

	_pw = None

	@classmethod
	def factory(cls):
		if cls._pw:
			return cls._pw
		cls._pw = cls()
		return cls._pw

	def __init__(self):
		# Generic window stuff
		gtk.Window.__init__(self)
		self.set_position(gtk.WIN_POS_CENTER)
		self.set_border_width(10)
		self.set_resizable(False)
		self.set_title("Zenoss Applet Preferences")

		# Get our configuration object
		osf = zapplet.os.OSFunctions.factory()

		# Close button
		sep   = gtk.HSeparator()
		close = gtk.Button(stock=gtk.STOCK_CLOSE)
		close.connect("clicked", lambda b: self.hide())

		# Server switcher / creator
		label  = gtk.Label("Zenoss Server:")
		combo  = gtk.combo_box_new_text()
		new    = gtk.Button(stock=gtk.STOCK_NEW)
		delete = gtk.Button(stock=gtk.STOCK_DELETE)
		entry  = gtk.Entry()
		add    = gtk.Button(stock=gtk.STOCK_ADD)
		cancel = gtk.Button(stock=gtk.STOCK_CANCEL)

		# Set default properties
		label.set_property("xalign", 1)
		combo.set_property("width-request", 250)
		delete.set_sensitive(False)
		entry.set_property("width-request", 251)
		entry.show()
		add.show()
		add.set_sensitive(False)
		cancel.show()

		# Setup the default servers from the config
		for s in sorted(osf.listServers()):
			combo.append_text(s)

		# Setup our preferences
		hostname = gtk.Entry()
		port     = gtk.SpinButton()
		ssl      = gtk.CheckButton("SSL")
		username = gtk.Entry()
		password = gtk.Entry()
		sound    = gtk.Button()
		interval = gtk.SpinButton()

		# Disable all preferences by default
		hostname.set_sensitive(False)
		port.set_sensitive(False)
		ssl.set_sensitive(False)
		username.set_sensitive(False)
		password.set_sensitive(False)
		sound.set_sensitive(False)
		interval.set_sensitive(False)

		# Set other misc widget prefs
		port.set_increments(1, 1)
		port.set_range(1, 65535)
		password.set_visibility(False)
		interval.set_increments(1, 1)
		interval.set_range(1, 60)

		# Create some HBoxes for packing neatness
		hp = gtk.HBox(spacing=5)
		hp.pack_start(hostname)
		hp.pack_start(gtk.Label("Port:"), False)
		ps = gtk.HBox(spacing=5)
		ps.pack_start(port)
		ps.pack_start(ssl, False)
		userpass = gtk.HBox(spacing=5)
		userpass.pack_start(username)
		userpass.pack_start(gtk.Label("Password:"), False)
		ins = gtk.HBox(spacing=5)
		ins.pack_start(interval, False)
		ins.pack_start(gtk.Label("minutes"), False)
		ins.pack_end(gtk.Label("Alert Sound:"), False)

		# Pack everything into the table
		table = gtk.Table(7, 4)
		table.set_row_spacings(10)
		self.add(table)
		table.attach(label,                       0, 1, 0, 1, gtk.FILL, gtk.FILL)
		table.attach(combo,                       1, 2, 0, 1, xpadding=5)
		table.attach(new,                         2, 3, 0, 1, gtk.FILL, gtk.FILL)
		table.attach(delete,                      3, 4, 0, 1, gtk.FILL, gtk.FILL)
		table.attach(gtk.HSeparator(),            0, 4, 1, 2)
		table.attach(gtk.Label("Hostname:"),      0, 1, 2, 3, gtk.FILL, gtk.FILL)
		table.attach(hp,                          1, 2, 2, 3, gtk.FILL, gtk.FILL, xpadding=5)
		table.attach(ps,                          2, 4, 2, 3, gtk.FILL, gtk.FILL)
		table.attach(gtk.Label("Username:"),      0, 1, 3, 4, gtk.FILL, gtk.FILL)
		table.attach(userpass,                    1, 2, 3, 4, gtk.FILL, gtk.FILL, xpadding=5)
		table.attach(password,                    2, 4, 3, 4, gtk.FILL, gtk.FILL)
		table.attach(gtk.Label("Poll Interval:"), 0, 1, 4, 5, gtk.FILL, gtk.FILL)		
		table.attach(ins,                         1, 2, 4, 5, xpadding=5)
		table.attach(sound,                       2, 4, 4, 5)
		table.attach(sep,                         0, 4, 5, 6, gtk.FILL, gtk.FILL)
		table.attach(close,                       3, 4, 6, 7, gtk.FILL, gtk.FILL)

		# Make sure all the labels are right aligned
		for child in table.get_children():
			if type(child) == gtk.Label:
				if "minutes" in child.get_text():
					child.set_alignment(0, 0.5)
				else:
					child.set_alignment(1, 0.5)

		def combo_changed(c):
			server = c and c.get_active_text() or None

			# Grey out the delete button if need be
			delete.set_sensitive(server != None)

			# Set values from new server
			if server:
				hostname.set_text (osf.get(server, "hostname"))
				port.set_value    (osf.get(server, "port"))
				ssl.set_active    (osf.get(server, "ssl"))
				username.set_text (osf.get(server, "username"))
				password.set_text (osf.get(server, "password"))
				interval.set_value(osf.get(server, "interval"))

			# Enable / Disable based upon server choice
			hostname.set_sensitive(server != None)
			port.set_sensitive(server != None)
			ssl.set_sensitive(server != None)
			username.set_sensitive(server != None)
			password.set_sensitive(server != None)
			#sound.set_sensitive(server != None)
			interval.set_sensitive(server != None)

		def new_clicked(b):
			table.remove(combo)
			table.remove(new)
			table.remove(delete)
			table.attach(entry,  1, 2, 0, 1, xpadding=5)
			table.attach(add,    2, 3, 0, 1, gtk.FILL, gtk.FILL)
			table.attach(cancel, 3, 4, 0, 1, gtk.FILL, gtk.FILL)
			entry.grab_focus()
			combo_changed(None)

		def delete_clicked(b):
			m = combo.get_model()
			t = combo.get_active_text()
			osf.delServer(t)
			m.remove(m.get_iter(filter(lambda r: r[0] == t, m)[0].path))
			if len(combo.get_model()) > 0:
				combo.set_active_iter(combo.get_model().get_iter(combo.get_model()[0].path))
			
		def cancel_clicked(b):
			entry.set_text("")
			table.remove(entry)
			table.remove(add)
			table.remove(cancel)
			table.attach(combo,  1, 2, 0, 1, xpadding=5)
			table.attach(new,    2, 3, 0, 1, gtk.FILL, gtk.FILL)
			table.attach(delete, 3, 4, 0, 1, gtk.FILL, gtk.FILL)
			combo_changed(combo)
	
		def add_clicked(b):
			m = combo.get_model()
			t = entry.get_text()
			osf.addServer(t)
			combo.append_text(t)
			combo.set_active_iter(m.get_iter(filter(lambda r: r[0] == t, m)[0].path))
			cancel_clicked(b)

		# Callbacks
		new.connect     ("clicked",       new_clicked)
		delete.connect  ("clicked",       delete_clicked)
		add.connect     ("clicked",       add_clicked)
		cancel.connect  ("clicked",       cancel_clicked)
		combo.connect   ("changed",       combo_changed)
		entry.connect   ("activate",      lambda e: add.clicked())
		entry.connect   ("changed",       lambda e: add.set_sensitive(e.get_text() != "" and not e.get_text() in [i[0] for i in combo.get_model()]))
		hostname.connect("changed",       lambda w: osf.set(combo.get_active_text(), "hostname", w.get_text()))
		port.connect    ("value-changed", lambda w: osf.set(combo.get_active_text(), "port",     w.get_value_as_int()))
		ssl.connect     ("toggled",       lambda w: osf.set(combo.get_active_text(), "ssl",      w.get_active()))
		username.connect("changed",       lambda w: osf.set(combo.get_active_text(), "username", w.get_text()))
		password.connect("changed",       lambda w: osf.set(combo.get_active_text(), "password", w.get_text()))
		interval.connect("value-changed", lambda w: osf.set(combo.get_active_text(), "interval", w.get_value_as_int()))

		# Select default server
		if len(combo.get_model()) > 0:
			combo.set_active_iter(combo.get_model().get_iter(combo.get_model()[0].path))

